#
# RecurringEvent/tests/testSetup.py
#
# Test RecurringEvent initialisation and set-up
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.RecurringEvent.tests import retc
from Products.RecurringEvent.config import RECUE_MODES, RECUE_OFFSETS
from Products.RecurringEvent.config import RM_ONE_TIME, RM_UNTIL, RM_FOREVER
from Products.RecurringEvent.config import RO_DAILY, RO_WEEKLY, RO_MONTHLY, RO_YEARLY
from DateTime import DateTime

class TestRecurring(retc.RecurringEventTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        self.css        = self.portal.portal_css
        self.kupu       = self.portal.kupu_library_tool
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.factory    = self.portal.portal_factory
        self.workflow   = self.portal.portal_workflow
        self.properties = self.portal.portal_properties

        self.metaTypes = ('RecurringEvent',)


    

class TestShifting(retc.RecurringEventTestCase):
    """Ensure shifting calcualtion goes well"""

    def afterSetUp(self):
        self.folder.invokeFactory('RecurringEvent', 're1')
        self.re1 = getattr(self.folder, 're1')

    def testCreateRecurringEvent(self):
        self.failUnless('re1' in self.folder.objectIds())
     
    def testDefaultValues(self):
        self.assertEqual(self.re1.defaultRecueMode(), RECUE_MODES.items()[0])
        self.assertEqual(self.re1.defaultRecueOffset(), RECUE_OFFSETS.items()[0])

    def testNumberOfDaysInMonth(self):
        dt = DateTime('2000/01/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/02/01')
        self.assertEqual(29, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2001/02/01')
        self.assertEqual(28, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/03/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/04/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/05/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/06/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/07/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/08/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/09/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/10/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))

        dt = DateTime('2000/11/01')
        self.assertEqual(30, self.re1.numberOfDaysInMonth(dt))
        
        dt = DateTime('2000/12/01')
        self.assertEqual(31, self.re1.numberOfDaysInMonth(dt))
        
        
    def testNumberOfDaysInYear(self):
        dt = DateTime('2000/12/01')
        self.assertEqual(366, self.re1.numberOfDaysInYear(dt))

        dt = DateTime('2001/12/01')
        self.assertEqual(365, self.re1.numberOfDaysInYear(dt))
        
    def testShiftDateTimeByOffset(self):
        self.re1.setRecueMode('until')
        self.re1.setRecueOffset('daily')
        
        #standardYear version
        dt = DateTime('2001/12/01')
        self.failUnless(DateTime('2001/12/02').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        #leapYear version
        dt = DateTime('2000/02/28')
        self.failUnless(DateTime('2000/02/29').equalTo(self.re1.shiftDateTimeByOffset(dt)))

        dt = DateTime('2000/12/01')
        self.re1.setRecueOffset('weekly')
        self.failUnless(DateTime('2000/12/08').equalTo(self.re1.shiftDateTimeByOffset(dt)))

		#leapYear version
        dt = DateTime('2000/02/01')
        self.re1.setRecueOffset('monthly')
        self.failUnless(DateTime('2000/03/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))

		#standardYear version
        dt = DateTime('2001/02/01')
        self.re1.setRecueOffset('monthly')
        self.failUnless(DateTime('2001/03/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))
		
		#year boundary
        self.re1.setRecueOffset('yearly')
		# additional leapYear testing
        dt = DateTime('2000/12/31')
        self.failUnless(dt.dayOfYear() == 366)
        
        dt = DateTime('1999/02/28')
        self.failUnless(DateTime('2000/02/28').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        # problem with extra day of 29'th Feb
        dt = DateTime('1999/03/01')
        self.failUnless(DateTime('2000/02/29').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        # shifts correctly becouse 2000 year has 366 days
        dt = DateTime('2000/02/28')
        self.failUnless(DateTime('2001/02/28').equalTo(self.re1.shiftDateTimeByOffset(dt)))
		
		
		# shifts incorrectly becouse date to be shifted is after 29'th Feb
        dt = DateTime('2000/03/01')
        self.failUnless(DateTime('2001/03/02').equalTo(self.re1.shiftDateTimeByOffset(dt)))

        dt = DateTime('2000/12/01')
        #leapYear version
        self.failUnless(dt.isLeapYear())
        self.failUnless(DateTime('2001/12/02').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
        #standardYear version
        dt = DateTime('2001/12/01')
        self.failUnless(not dt.isLeapYear())
        self.failUnless(DateTime('2002/12/01').equalTo(self.re1.shiftDateTimeByOffset(dt)))
        
    def testRecueDays(self):
        self.re1.setStartDate(DateTime('2001/01/01 12:30'))
        self.re1.setRecueUntilDate(DateTime('2001/01/31 5:20'))
        
        self.re1.setRecueMode(RM_UNTIL)
        self.re1.setRecueOffset(RO_DAILY)
       
        days = self.re1.getRecueDays()
       
        self.failUnless(len(days) == 31)      
         
    def testRecueWeeks(self):
        self.re1.setStartDate(DateTime('2006/01/01 12:30'))
        self.re1.setRecueUntilDate(DateTime('2006/01/31 5:20'))
        
        self.re1.setRecueMode(RM_UNTIL)
        self.re1.setRecueOffset(RO_DAILY)
       
        weeks = self.re1.getRecueWeeks()
        self.failUnless(len(weeks) == 5)       
        
        self.re1.setStartDate(DateTime('2006/01/01 12:30'))
        self.re1.setRecueUntilDate(DateTime('2006/12/31 5:20'))
        weeks = self.re1.getRecueWeeks()
        self.failUnless(len(weeks) == 53)       
        
        self.re1.setRecueUntilDate(DateTime('2016/12/31 5:20'))
        weeks = self.re1.getRecueWeeks()

        # there is almost 11 years and 2 shared weeks: one at start, second at end
        self.failUnless(len(weeks) == 11*52+2)       

    def testRecueMonths(self):
        self.re1.setStartDate(DateTime('2001/01/01 12:30'))
        self.re1.setRecueUntilDate(DateTime('2001/01/31 5:20'))
        
        self.re1.setRecueMode(RM_UNTIL)
        self.re1.setRecueOffset(RO_DAILY)
       
        months = self.re1.getRecueMonths()
        self.failUnless(len(months) == 1)    

        self.re1.setRecueUntilDate(DateTime('2010/01/31 5:20'))
        months = self.re1.getRecueMonths()
        self.failUnless(len(months) == 10*12 - 11)    

        self.re1.setRecueUntilDate(DateTime('2010/12/31 5:20'))
        months = self.re1.getRecueMonths()
        self.failUnless(len(months) == 10*12)    

    def testRecueYears(self):
        self.re1.setStartDate(DateTime('2001/01/01 12:30'))
        self.re1.setRecueUntilDate(DateTime('2001/01/31 5:20'))
        
        self.re1.setRecueMode(RM_UNTIL)
        self.re1.setRecueOffset(RO_DAILY)
       
        years = self.re1.getRecueYears()
       
        self.failUnless(len(years) == 1) 
                 

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestContentCreation))
    return suite
